<?php

class GC_PaypalValidateModuleFrontController extends ModuleFrontController {
	
	public $use_curl = true; // Set to false to execute IPN by file_get_contents() instead of cUrl
	public $debug_response = false; // Set to true to get some debug informations in /modules/gc_paypal/ipn_error.txt
	
	function postProcess() {
		
		parent::postProcess();
		
		$raw_post_data  = file_get_contents('php://input');
		$raw_post_array = explode('&', $raw_post_data);
		$myPost         = array();
		
		foreach ($raw_post_array as $keyval) {
			
			$keyval = explode ('=', $keyval);
			
			if(count($keyval) == 2)
				$myPost[$keyval[0]] = urldecode($keyval[1]);
			
		}
		
		$req = 'cmd=_notify-validate';
		
		if(function_exists('get_magic_quotes_gpc')) {
			$get_magic_quotes_exists = true;
		}
		
		foreach ($myPost as $key => $value) {        
			
			if($get_magic_quotes_exists == true && get_magic_quotes_gpc() == 1) { 
				$value = urlencode(stripslashes($value)); 
			} else {
				$value = urlencode($value);
			}
			
			$req .= "&$key=$value";
			
		}
		
		$paypal_url = Configuration::get('GC_PAYPAL_MODE') ? 'https://www.paypal.com/cgi-bin/webscr' : 'https://www.sandbox.paypal.com/cgi-bin/webscr';
		
		if($this->use_curl) {
			
			$ch = curl_init();
			
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, true);
			curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
			curl_setopt($ch, CURLOPT_URL,            $paypal_url);
			curl_setopt($ch, CURLOPT_POST,           true);
			curl_setopt($ch, CURLOPT_POSTFIELDS,     $req);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			
			$res = curl_exec($ch);
			$status = strval(curl_getinfo($ch, CURLINFO_HTTP_CODE));
			
			if(empty($res) || $status == '0') {
				
				file_put_contents(
					dirname(__FILE__).'/../../ipn_error.txt', 
					"cUrl Errno [".curl_errno($ch)."]\nHTTP Status [".$status."]\ncUrl Error: ".curl_error($ch)."\n"/*.print_r(curl_getinfo($ch), true)*/
				);
				
				curl_close($ch);
				
				die();
				
			}
			
			curl_close($ch);
			
		}
		else {
			
			$res = file_get_contents($paypal_url.'?'.$req);
			
		}
		
		if($this->debug_response)
			file_put_contents(
				dirname(__FILE__).'/../../ipn_error.txt', 
				"Invoice [".$_POST['invoice']."]\n".$res
			);
		
		if(!empty($res) and isset($_POST['invoice']) and !empty($_POST['invoice'])) {
			
			$related_orders_collection = Order::getByReference($_POST['invoice']);
			$related_orders = $related_orders_collection->getAll();
			$first_order = $related_orders_collection->getFirst();
			
			$return_amount = (float)$_POST['mc_gross'];
			$order_amount = (float)$first_order->getOrdersTotalPaid();
			
			$payment_status = $_POST['payment_status'];
			
			$error_message = '';
			
			foreach($related_orders as $related_order) {
				
				if (stripos($res, "VERIFIED") !== false) {
					
					if($return_amount == $order_amount) {
						switch($payment_status) {
							
							case 'Completed':
							case 'Processed':
								$related_order->setCurrentState(Configuration::get('PS_OS_PAYMENT'));
								break;
							case 'Denied':
							case 'Expired':
							case 'Failed':
							case 'Voided':
								$related_order->setCurrentState(Configuration::get('PS_OS_ERROR'));
								$error_message = 'PayPal IPN Payment Status: '.$payment_status;
								break;
							
						}
						
					}
					else {
						$related_order->setCurrentState(Configuration::get('PS_OS_ERROR'));
						$error_message = 'PayPal IPN Amount: '.$return_amount.' ('.$order_amount.')';
					}
					
				}
				elseif(stripos($res, "INVALID") !== false) {
					$related_order->setCurrentState(Configuration::get('PS_OS_ERROR'));
					$error_message = 'PayPal IPN Response: INVALID';
				}
				
			}
			
			if(!empty($error_message)) {
				
				$customer = new Customer($related_order->id_customer);
				
				$id_customer_thread = CustomerThread::getIdCustomerThreadByEmailAndIdOrder($customer->email, $related_order->id);
				
				if (!$id_customer_thread)
				{
					$customer_thread = new CustomerThread();
					$customer_thread->id_contact  = 0;
					$customer_thread->id_customer = (int)$related_order->id_customer;
					$customer_thread->id_shop     = (int)$related_order->id_shop;
					$customer_thread->id_order    = (int)$related_order->id;
					$customer_thread->id_lang     = (int)$related_order->id_lang;
					$customer_thread->email       = $customer->email;
					$customer_thread->status      = 'open';
					$customer_thread->token       = Tools::passwdGen(12);
					$customer_thread->add();
				}
				else
					$customer_thread = new CustomerThread((int)$id_customer_thread);
				
				$customer_message = new CustomerMessage();
				$customer_message->id_customer_thread = $customer_thread->id;
				$customer_message->id_employee = 0;
				$customer_message->message = $_POST['invoice'].' | '.$error_message;
				$customer_message->private = 1;
				$customer_message->add();
			}
			
		}
		
		die();
		
	}
	
}