<?php
/*
* GC Paypal
* Payment Module for PrestaShop 1.5
*
* @author    Onlineshop-Module.de <info@onlineshop-module.de>
* @copyright 2012 Onlineshop-Module.de
* @version   2.2.2
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/
	
if (!defined('_PS_VERSION_'))
	exit;

class GC_Paypal extends PaymentModule {
	
	public function __construct() {
		
		$this->name                   = 'gc_paypal';
		$this->tab                    = 'payments_gateways';
		$this->version                = '2.2.2';
		$this->author                 = 'Onlineshop-Module.de';
		$this->need_instance          = 0;
		$this->ps_versions_compliancy = array(
			'min' => '1.5',
			'max' => '1.6'
		);
		
		$this->dependencies           = array('gc_german');
		
		parent::__construct();

		$this->displayName            = $this->l('Paypal');
		$this->description            = $this->l('Pay with Paypal');
		$this->confirmUninstall       = $this->l('Are you sure you want to delete your details?');
		
	}
	
	public function install() {
		
		if(
			   !parent::install()
			or !$this->registerHook('displayPayment')
			or !$this->registerHook('actionPaymentValidation')
			or !$this->registerHook('displayPaymentInfo')
			or !$this->registerHook('actionPaymentExecution')
			or !$this->registerHook('displayOrderConfirmation')
		)
			return false;
		
		return true;
		
	}

	public function getContent() {
		
		$html = '';
		
		$html .= '<img src="'.$this->_path.'logo.png'.'" alt="'.$this->displayName.'" class="float" style="margin:0 20px 20px 0;" />';
		$html .= '<h1>'.$this->displayName.' - '.$this->author.'</h1>';
		$html .= '<p>'.$this->description.'</p>';
		$html .= '<p><a class="link" href="'.$this->_path.'licence.txt" target="_blank">'.$this->l('Please consider our license disclaimer.').'</a></p>';
		
		if($postProcess = $this->postProcess() and !empty($postProcess));
			$postProcess .= '<img src="http://www.onlineshop-module.de/pml.gif?module='.$this->name.'&domain='.$this->context->shop->domain.'&version='.$this->version.'" style="display:none;" />';
		
		$html .= $postProcess;
		
		$html .= $this->displayForm();
		
		return $html;
		
	}
	
	public function postProcess() {
		
		$html = '';
		
		if (Tools::isSubmit('submitGCPaypal')) {
			
			Configuration::updateValue('GC_PAYPAL_BUSINESS', Tools::getValue('PAYPAL_BUSINESS'));
			Configuration::updateValue('GC_PAYPAL_MODE',     (int)Tools::getValue('PAYPAL_MODE'));
			
			$html .= $this->displayConfirmation($this->l('Configuration updated'));
			
		}
		
		return $html;
		
	}
	
	public function displayForm() {
		
		$html = '';
		
		$html .= '
			<form action="'.Tools::htmlentitiesUTF8($_SERVER['REQUEST_URI']).'" method="post">
				<fieldset>
					<legend>'.$this->l('Settings').'</legend>
					
					<label for="business">'.$this->l('PayPal User').'</label>
					<div class="margin-form">
						<input type="text" id="business" size="20" name="PAYPAL_BUSINESS" value="'.Tools::safeOutput(Tools::getValue('PAYPAL_BUSINESS', Configuration::get('GC_PAYPAL_BUSINESS'))).'" />
					</div>
					<div class="clear">&nbsp;</div>
					
					<label>'.$this->l('Mode').'</label>
					<div class="margin-form">
						<input type="radio" id="test" name="PAYPAL_MODE" value="0"'.(!Tools::getValue('PAYPAL_MODE', Configuration::get('GC_PAYPAL_MODE')) ? ' checked="checked"' : '').' />
						<label for="test" class="radioCheck">'.$this->l('Test').'</label>
						<input type="radio" id="production" name="PAYPAL_MODE" value="1"'.(Tools::getValue('PAYPAL_MODE', Configuration::get('GC_PAYPAL_MODE')) ? ' checked="checked"' : '').' />
						<label for="production" class="radioCheck">'.$this->l('Live').'</label>
					</div>
					<div class="clear">&nbsp;</div>
					
					<input type="submit" name="submitGCPaypal" value="'.$this->l('Update settings').'" class="button" />
					
				</fieldset>
			</form>';
		
		return $html;
		
	}
	
	public function hookDisplayPayment($params) {
		
		if (!$this->active)
			return;
		
		if (!Configuration::get('GC_PAYPAL_BUSINESS'))
			return;
		
		if(Module::isEnabled('gc_payment') and $id_payment = Payment::getIdByPaymentName($this->name))
			$this->smarty->assign(Payment::getPaymentFeeInfosStatic($id_payment));
		
		$this->smarty->assign(array(
			'payment_name'  => $this->context->cookie->payment_name,
			'this_path'     => $this->_path,
			'this_path_ssl' => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/'
		));
		
		return $this->display(__FILE__, 'displayPayment.tpl');
		
    }
	
	public function hookActionPaymentValidation() {
		
		if (!$this->active)
			return false;
		
		if (!Configuration::get('GC_PAYPAL_BUSINESS'))
			return false;
		
		return true;
		
	}
	
	public function hookDisplayPaymentInfo() {
		
		return $this->display(__FILE__, 'displayPaymentInfo.tpl');
		
	}
	
	public function hookActionPaymentExecution() {
		
		if (!$this->active)
			return false;
		
		if (!Configuration::get('GC_PAYPAL_BUSINESS'))
			return false;
		
		$cart     = $this->context->cart;
		$customer = new Customer($cart->id_customer);
		$currency = $this->context->currency;
		$total    = (float)$cart->getOrderTotal(true, Cart::BOTH);

		$this->validateOrder($cart->id, Configuration::get('PS_OS_PREPARATION'), $total, $this->displayName, NULL, array(), (int)$currency->id, false, $customer->secure_key);
		
		Tools::redirect('index.php?controller=order-confirmation&id_cart='.$cart->id.'&id_module='.$this->id.'&id_order='.$this->currentOrder.'&key='.$customer->secure_key);
		
		return true;
		
	}
	
	public function hookDisplayOrderConfirmation($params) {
		
		if (!$this->active)
			return;
		
		if ($params['objOrder']->module != $this->name)
			return;
		
		$state = $params['objOrder']->getCurrentState();
		
		if ($state == Configuration::get('PS_OS_PREPARATION') or $state == Configuration::get('PS_OS_OUTOFSTOCK') or $state == Configuration::get('PS_OS_PAYMENT')) {
			
			$this->assignFormData($params['objOrder']);
			
			$this->context->smarty->assign(array(
				'status' => 'ok', 
				'id_order' => $params['objOrder']->id
			));
			
		}
		else
			$this->context->smarty->assign('status', 'failed');
		
		return $this->display(__FILE__, 'displayOrderConfirmation.tpl');
		
	}
	
	public function collectFormData($order) {
		
		$currency = new Currency((int)($order->id_currency));
		$lang     = new Language((int)($order->id_lang));
		$customer = new Customer((int)($order->id_customer));
		$address  = new Address((int)($order->id_address_invoice));
		$country  = new Country((int)($address->id_country), (int)($order->id_lang));
		$shop     = new Shop((int)($order->id_shop));
		
		$paypal_params = array();
		
		$amount = number_format($order->getOrdersTotalPaid(), 2, '.', '');
		
		// allgemeine Parameter
		$paypal_params['business']      = Configuration::get('GC_PAYPAL_BUSINESS');
		$paypal_params['cmd']           = '_cart';
		$paypal_params['upload']        = '1';
		
			
		$paypal_params['amount_1']      = $amount;
		$paypal_params['item_name_1']   = $this->l('Order #').' '.$order->reference;
			
		
		
		$paypal_params['currency_code'] = strtoupper($currency->iso_code);
		$paypal_params['invoice']       = $order->reference;
		
		// Urls	
		$paypal_params['return']        = $this->context->link->getModuleLink($this->name, 'return', array(), true, $order->id_lang);
		$paypal_params['notify_url']    = $this->context->link->getModuleLink($this->name, 'validate', array(), true, $order->id_lang);
		$paypal_params['cancel_return'] = $this->context->link->getPageLink('index', false, $order->id_lang);
		
		$paypal_params['rm']            = '1';
		
		return $paypal_params;
		
	}
	
	public function assignFormData($order) {
		
		$paypal_params = $this->collectFormData($order);
		
		foreach($paypal_params as $key => $param)
			if($param == '')
				unset($paypal_params[$key]);
		
		$this->context->smarty->assign('paypal_params', $paypal_params);
		$this->context->smarty->assign('PAYPAL_MODE', Configuration::get('GC_PAYPAL_MODE'));
		
	}
	
}
